﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {
  [StorableClass]
  [Item("InvertedGenerationalDistanceAnalyzer", "The inverted generational distance between the current and the best known front (see Multi-Objective Performance Metrics - Shodhganga for more information)")]
  public class InvertedGenerationalDistanceAnalyzer : MOTFAnalyzer {
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
    }
    [StorableConstructor]
    protected InvertedGenerationalDistanceAnalyzer(bool deserializing) : base(deserializing) { }
    public InvertedGenerationalDistanceAnalyzer(InvertedGenerationalDistanceAnalyzer original, Cloner cloner) : base(original, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new InvertedGenerationalDistanceAnalyzer(this, cloner);
    }

    /// <summary> 
    /// </summary>
    private IValueParameter<DoubleValue> DampeningParameter {
      get {
        return (IValueParameter<DoubleValue>)Parameters["Dampening"];
      }
      set {
        Parameters["Dampening"].ActualValue = value;
      }
    }

    public InvertedGenerationalDistanceAnalyzer() {
      Parameters.Add(new ValueParameter<DoubleValue>("Dampening", "", new DoubleValue(1)));
    }

    public override void Analyze(Individual[] individuals, double[][] qualities, ResultCollection results) {
      int objectives = qualities[0].Length;
      var optimalfront = TestFunctionParameter.ActualValue.OptimalParetoFront(objectives);
      if (optimalfront == null) {
        return;
      }
      if (!results.ContainsKey("InvertedGenerationalDistance")) results.Add(new Result("InvertedGenerationalDistance", typeof(DoubleValue)));
      results["InvertedGenerationalDistance"].Value = new DoubleValue(InvertedGenerationalDistance.Calculate(qualities, optimalfront, DampeningParameter.Value.Value));
    }
  }
}
